# Copyright (c) 2008-2014 Michael Howitz
# See also LICENSE.txt
"""I18n message factory for icemac.addressbook. """


import zope.i18nmessageid


_ = MessageFactory = zope.i18nmessageid.MessageFactory('icemac.addressbook')
